#include "Recorder.h"

#define BUFFER_SIZE   1024*1024

Recorder::Recorder(const wchar_t* filename, const int mode, const int framewidth, const int frameheight) : file(NULL), buffer(NULL), bufflen(0), buffpos(0), mode(mode)
{
  if (mode==RECORDER_RECORD)
  {
    if (!_wfopen_s(&file, filename, L"wb") && file)
    {
      buffpos=0;
      bufflen=BUFFER_SIZE;
      buffer=(BYTE*) malloc(BUFFER_SIZE);

      fwrite(&framewidth, sizeof(int), 1, file);
      fwrite(&frameheight, sizeof(int), 1, file);
#ifdef NEAR_MODE
      int nearmode=1;
#else
      int nearmode=0;
#endif
      fwrite(&nearmode, sizeof(int), 1, file);
    }
  }
  else
  {
    if (!_wfopen_s(&file, filename, L"rb") && file)
    {
      int t1, t2, nearmode;
      fread(&t1, sizeof(int), 1, file);
      fread(&t2, sizeof(int), 1, file);
      fread(&nearmode, sizeof(int), 1, file);

#ifdef NEAR_MODE
      bool valid=(nearmode==1);
#else
      bool valid=(nearmode==0);
#endif
      
      if (valid && t1==framewidth && t2==frameheight)
      {
        buffpos=0;
        bufflen=20;//BUFFER_SIZE;
        buffer=(BYTE*) malloc(BUFFER_SIZE);
        fread(buffer, 4, 1, file);
      }
      else
      {
        fclose(file);
        file=NULL;
      }
    }
  }
}

Recorder::~Recorder() 
{
  CloseRecorder();
}

void Recorder::WritePoint(const POINTDATA* point, const DWORD index)
{
  if (mode!=RECORDER_RECORD)
    return;

  if (buffpos+20>bufflen)
  {
    int written=fwrite(buffer, 1, buffpos, file);
    while (written<buffpos)
    {
      int extra=fwrite(buffer+written, 1, buffpos-written, file);
      if (extra==0)
        break;
      else
        written+=extra;
    } 
    buffpos=0;
  }

  *(DWORD*) (buffer+buffpos)=index;
/*  *(float*) (buffer+buffpos+4)=point->colour[0];
  *(float*) (buffer+buffpos+8)=point->colour[1];
  *(float*) (buffer+buffpos+12)=point->colour[2];*/
  *(float*) (buffer+buffpos+16)=point->xyz[2];

  buffpos+=20;
//  fwrite(buffer, 1, buffpos, file);
//  buffpos=0;
}

int Recorder::GetFrameData(POINTDATA* point, const int width, const int height)
{
  if (mode!=RECORDER_PLAYBACK)
    return 0;

  int used=0;
  DWORD index, lastindex=0;
  DWORD max=width*height;
  for (index=0; index<max; index++)
  {
//    point[index].paired=false;
    point[index].used=false;
  }

  index=*(DWORD*) buffer;
  while (index>=lastindex && index<max)
  {
    if (fread(buffer+4, 1, 16, file)==16)
    {
      point[index].used=true;
/*      point[index].colour[0]=*(float*) (buffer+4);
      point[index].colour[1]=*(float*) (buffer+8);
      point[index].colour[2]=*(float*) (buffer+12);*/
      point[index].xyz[2]=*(float*) (buffer+16);
      ++used;

      lastindex=index;

      if (fread(buffer, 4, 1, file)==1)
        index=*(DWORD*) buffer;
      else
      {
        *(DWORD*) buffer=0xFFFFFFFF;
        index=0xFFFFFFFF;
      }
    }
    else
      index=0xFFFFFFFF;
  }

  return used;
}

/*
void Recorder::WriteData(const BYTE* data, const int offset, const int len)
{
  if (buffpos+len>bufflen)
  {
    fwrite(buffer, 1, buffpos, file);
    if (len<BUFFER_SIZE/8)
    {
      memcpy(buffer, data+offset, len);
      buffpos=len;
    }
    else
    {
      fwrite(data+offset, 1, len, file);
      buffpos=0;
    }
  }
  else
  {
    memcpy(buffer+buffpos, data+offset, len);
    buffpos+=len;
  }
}*/

void Recorder::CloseRecorder()
{
  if (file)
  {
    if (mode==RECORDER_RECORD)
      fwrite(buffer, 1, buffpos, file);
    fclose(file);
    file=NULL;
    free(buffer);
    bufflen=0;
    buffpos=0;
  }
}