#include <iostream>
#include <pcl/io/pcd_io.h>
#include <pcl/point_types.h>
#include <pcl/registration/icp.h>

#include "Recorder.h"

#define WIDTH   320
#define HEIGHT  240
#define WORLD_SCALE           0.02f // x and y difference across pixels

void ExampleCode();

int main(int argc, char** argv)
{
//  ExampleCode();
//  return 0;

  Recorder recorder(L"Record-pan.dat", RECORDER_PLAYBACK, WIDTH, HEIGHT);

  POINTDATA* pointdata=(POINTDATA*) malloc(WIDTH*HEIGHT*sizeof(POINTDATA));
  int pos=0;
  for (int j=0; j<HEIGHT; j++)
  {
    for (int i=0; i<WIDTH; i++)
    {
      pointdata[pos].xyz[0]=(i-(WIDTH/2))*WORLD_SCALE;
      pointdata[pos++].xyz[1]=(j-(HEIGHT/2))*WORLD_SCALE;
    }
  }

  pcl::PointCloud<pcl::PointXYZ>::Ptr cloud_in (new pcl::PointCloud<pcl::PointXYZ>);
  pcl::PointCloud<pcl::PointXYZ>::Ptr cloud_out (new pcl::PointCloud<pcl::PointXYZ>);

  int size=recorder.GetFrameData(pointdata, WIDTH, HEIGHT);
  // Fill in the CloudIn data
  cloud_out->width    = size;
  cloud_out->height   = 1;
  cloud_out->is_dense = true;
  cloud_out->points.resize(size);

  pos=0;
  for (size_t i=0; i<WIDTH*HEIGHT; ++i)
  {
    if (pointdata[i].used)
    {
      cloud_out->points[pos].x=pointdata[i].xyz[0];
      cloud_out->points[pos].y=pointdata[i].xyz[1];
      cloud_out->points[pos++].z=pointdata[i].xyz[2];
    }
  }
  std::cout << "Saved " << cloud_out->points.size() << " data points to target:" << std::endl;

  int frame=0;
  Eigen::Matrix4f guess=Eigen::Matrix4f::Identity();

  while (frame<200)
  {
    size=recorder.GetFrameData(pointdata, WIDTH, HEIGHT);
    // Fill in the CloudIn data
    cloud_in->width    = size;
    cloud_in->height   = 1;
    cloud_in->is_dense = true;
    cloud_in->points.resize(size);

    pos=0;
    for (size_t i=0; i<WIDTH*HEIGHT; ++i)
    {
      if (pointdata[i].used)
      {
        cloud_in->points[pos].x=pointdata[i].xyz[0];
        cloud_in->points[pos].y=pointdata[i].xyz[1];
        cloud_in->points[pos++].z=pointdata[i].xyz[2];
      }
    }

  //  *cloud_in=*cloud_out;
    std::cout << "Frame: " << frame << " - Points " << cloud_in->points.size() << " data points to input:" << std::endl;

    pcl::IterativeClosestPoint<pcl::PointXYZ, pcl::PointXYZ> icp;
    icp.setInputCloud(cloud_in);
    icp.setInputTarget(cloud_out);
    pcl::PointCloud<pcl::PointXYZ> Final;
    icp.align(Final, guess);
  
    std::cout << "has converged:" << icp.hasConverged() << " score: " << icp.getFitnessScore() << std::endl;
  
    guess=icp.getFinalTransformation();
    std::cout << guess << std::endl;
    ++frame;
  }

  return (0);
}

void ExampleCode()
{
  pcl::PointCloud<pcl::PointXYZ>::Ptr cloud_in (new pcl::PointCloud<pcl::PointXYZ>);
  pcl::PointCloud<pcl::PointXYZ>::Ptr cloud_out (new pcl::PointCloud<pcl::PointXYZ>);

  // Fill in the CloudIn data
  cloud_in->width    = 50000;
  cloud_in->height   = 1;
  cloud_in->is_dense = true;
  
  cloud_in->points.resize (cloud_in->width * cloud_in->height);
  for (size_t i = 0; i < cloud_in->points.size (); ++i)
  {
    cloud_in->points[i].x = 1024 * rand () / (RAND_MAX + 1.0f);
    cloud_in->points[i].y = 1024 * rand () / (RAND_MAX + 1.0f);
    cloud_in->points[i].z = 1024 * rand () / (RAND_MAX + 1.0f);
  }
  
  std::cout << "Saved " << cloud_in->points.size () << " data points to input:" << std::endl;
  
//  for (size_t i = 0; i < cloud_in->points.size (); ++i) 
//    std::cout << "    " << cloud_in->points[i].x << " " << cloud_in->points[i].y << " " << cloud_in->points[i].z << std::endl;
  
  *cloud_out = *cloud_in;
  
  std::cout << "size:" << cloud_out->points.size() << std::endl;
  for (size_t i = 0; i < cloud_in->points.size (); ++i)
    cloud_out->points[i].x = cloud_in->points[i].x + 0.7f;
  
  std::cout << "Transformed " << cloud_in->points.size () << " data points:" << std::endl;
  
//  for (size_t i = 0; i < cloud_out->points.size (); ++i)
//    std::cout << "    " << cloud_out->points[i].x << " " << cloud_out->points[i].y << " " << cloud_out->points[i].z << std::endl;

  pcl::IterativeClosestPoint<pcl::PointXYZ, pcl::PointXYZ> icp;
  icp.setInputCloud(cloud_in);
  icp.setInputTarget(cloud_out);
  pcl::PointCloud<pcl::PointXYZ> Final;
  icp.align(Final);
  std::cout << "has converged:" << icp.hasConverged() << " score: " <<
  icp.getFitnessScore() << std::endl;
  std::cout << icp.getFinalTransformation() << std::endl;

}